
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		Executables							#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

CC	=	gcc
LD	=	gcc-ld
GEN	=	xgen.ttp

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		Compiler mode							#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

CMODE	=
#		-Wall

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		Target machines							#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

T040	=	-m68040 -m68881
T881	=	-m68030 -m68881
T030	=	-m68030
T68K	=	-m68000

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		Optimisations							#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

O040	=	-O3 \
		-fomit-frame-pointer \
		-fcaller-saves \
		-fdefer-pop \
		-fthread-jumps \
		-fcse-follow-jumps \
		-fcse-skip-blocks \
		-frerun-cse-after-loop \
		-fstrength-reduce \
		-fpeephole \
		-ffast-math \
		-fno-inline-functions \
		-fno-unroll-loops \
		-fno-expensive-optimizations \
		-fno-force-addr \
		-fno-force-mem \

O881	=	-O3 \
		-fomit-frame-pointer \
		-fcaller-saves \
		-fdefer-pop \
		-fthread-jumps \
		-fcse-follow-jumps \
		-fcse-skip-blocks \
		-frerun-cse-after-loop \
		-fstrength-reduce \
		-fpeephole \
		-ffast-math \
		-finline-functions \
		-fno-unroll-loops \
		-fno-expensive-optimizations \
		-fno-force-addr \
		-fno-force-mem \

O030	=	-O3 \
		-fomit-frame-pointer \
		-fcaller-saves \
		-fdefer-pop \
		-fthread-jumps \
		-fcse-follow-jumps \
		-fcse-skip-blocks \
		-frerun-cse-after-loop \
		-fstrength-reduce \
		-fpeephole \
		-ffast-math \
		-finline-functions \
		-fno-unroll-loops \
		-fno-expensive-optimizations \
		-fno-force-addr \
		-fno-force-mem \

O68K	=	-O3 \
		-fomit-frame-pointer \
		-fcaller-saves \
		-fdefer-pop \
		-fthread-jumps \
		-fcse-follow-jumps \
		-fcse-skip-blocks \
		-frerun-cse-after-loop \
		-fstrength-reduce \
		-fpeephole \
		-ffast-math \
		-finline-functions \
		-funroll-loops \
		-fno-expensive-optimizations \
		-fno-force-addr \
		-fno-force-mem \

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		Project libraries & link files					#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

L040	=	-lpng040 -lzlib040 -lgnu040
L881	=	-lpng881 -lzlib030 -lgnu881
L030	=	-lpml030 -lpng030 -lzlib030 -lgnu030
LSFP	=	-lpmlsfp -lpng -lzlib -lgnu
L68K	=	-lpml -lpng -lzlib -lgnu

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		GEN assembler flags						#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

GFLAGS	=

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		GAS assembler flags						#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

AFLAGS	=

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		Linker flags							#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

LFLAGS	=

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		Project description						#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

CTARG	=	$(T040)
COPTI	=	$(O040)
LIBS	=	$(L040)
OUT	=	pov3_040.ttp

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		Object files							#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

COBJ	=	povray.o mem.o render.o ray.o matrices.o camera.o pigment.o \
		normal.o colour.o texture.o lighting.o csg.o vbuffer.o \
		vlbuffer.o lbuffer.o bbox.o bsphere.o objects.o spheres.o \
		boxes.o planes.o blob.o torus.o triangle.o cones.o hfield.o \
		lathe.o sor.o truetype.o polygon.o prism.o mesh.o discs.o \
		quadrics.o polysolv.o fractal.o hcmplx.o bezier.o atmosph.o \
		pattern.o quatern.o halos.o point.o txttest.o chi2.o warps.o \
		octree.o radiosit.o rad_data.o targa.o png_pov.o ppm.o pgm.o \
		iff.o gif.o gifdecod.o image.o parse.o express.o tokenize.o \
		parstxtr.o optin.o optout.o userio.o port.o poly.o super.o \

AOBJ	=	videl.o \

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		Project rules							#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

$(OUT):		$(COBJ) $(AOBJ)
		$(LD) $(LFLAGS) -o$(OUT) $(GNULIB)\crt0.o $(COBJ) $(AOBJ) $(LIBS)
		fixstk 131072 $(OUT)
		strip $(OUT)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		Dependency rules for object files				#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

warps.o:	warps.c frame.h config.h vector.h povproto.h mem.h matrices.h warps.h \
		pattern.h texture.h 

vlbuffer.o:	vlbuffer.c frame.h config.h vector.h povproto.h mem.h bbox.h vlbuffer.h 

vbuffer.o:	vbuffer.c frame.h config.h vector.h povproto.h mem.h bbox.h boxes.h \
		hfield.h lighting.h point.h vlbuffer.h matrices.h objects.h povray.h atmosph.h \
		warps.h camera.h render.h triangle.h vbuffer.h userio.h 

userio.o:	userio.c frame.h config.h vector.h povproto.h mem.h parse.h povray.h \
		atmosph.h warps.h camera.h point.h vlbuffer.h bbox.h render.h tokenize.h userio.h 

txttest.o:	txttest.c frame.h config.h vector.h povproto.h mem.h texture.h pattern.h \
		warps.h povray.h atmosph.h camera.h point.h vlbuffer.h bbox.h render.h txttest.h 

truetype.o:	truetype.c frame.h config.h povray.h atmosph.h warps.h camera.h \
		point.h vlbuffer.h bbox.h render.h vector.h povproto.h mem.h matrices.h objects.h \
		truetype.h csg.h 

triangle.o:	triangle.c frame.h config.h povray.h atmosph.h warps.h camera.h \
		point.h vlbuffer.h bbox.h render.h vector.h povproto.h mem.h matrices.h objects.h \
		triangle.h 

torus.o:	torus.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h polysolv.h matrices.h objects.h \
		torus.h 

tokenize.o:	tokenize.c frame.h config.h povray.h atmosph.h warps.h camera.h \
		point.h vlbuffer.h bbox.h render.h povproto.h mem.h parse.h tokenize.h express.h 

texture.o:	texture.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h texture.h pattern.h halos.h \
		image.h matrices.h normal.h pigment.h 

targa.o:	targa.c frame.h config.h povproto.h mem.h povray.h atmosph.h warps.h \
		camera.h point.h vlbuffer.h bbox.h render.h targa.h 

super.o:	super.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h matrices.h objects.h super.h 

spheres.o:	spheres.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h matrices.h objects.h spheres.h 

sor.o:		sor.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h vlbuffer.h \
		bbox.h render.h vector.h povproto.h mem.h polysolv.h matrices.h objects.h sor.h 

render.o:	render.c frame.h config.h vector.h povproto.h mem.h bbox.h chi2.h \
		colour.h lighting.h point.h vlbuffer.h normal.h objects.h octree.h optout.h \
		povray.h atmosph.h warps.h camera.h render.h radiosit.h ray.h targa.h texture.h \
		pattern.h vbuffer.h userio.h 

ray.o:		ray.c frame.h config.h vector.h povproto.h mem.h povray.h atmosph.h \
		warps.h camera.h point.h vlbuffer.h bbox.h render.h ray.h texture.h pattern.h 

rad_data.o:	rad_data.c frame.h config.h octree.h radiosit.h 

radiosit.o:	radiosit.c frame.h config.h lighting.h point.h vlbuffer.h bbox.h \
		vector.h povray.h atmosph.h warps.h camera.h render.h optin.h povproto.h mem.h \
		texture.h pattern.h octree.h radiosit.h ray.h 

quatern.o:	quatern.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h fractal.h quatern.h spheres.h 

quadrics.o:	quadrics.c frame.h config.h povray.h atmosph.h warps.h camera.h \
		point.h vlbuffer.h bbox.h render.h vector.h povproto.h mem.h objects.h matrices.h \
		planes.h quadrics.h 

prism.o:	prism.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h matrices.h objects.h polysolv.h \
		prism.h 

ppm.o:		ppm.c frame.h config.h povproto.h mem.h povray.h atmosph.h warps.h camera.h \
		point.h vlbuffer.h bbox.h render.h optout.h pgm.h ppm.h 

povray.o:	povray.c frame.h config.h povproto.h mem.h bezier.h blob.h bsphere.h \
		bbox.h cones.h csg.h discs.h express.h fractal.h hfield.h boxes.h lathe.h lighting.h \
		point.h vlbuffer.h mesh.h polysolv.h objects.h octree.h parse.h pigment.h poly.h \
		polygon.h povray.h atmosph.h warps.h camera.h render.h optin.h optout.h quadrics.h \
		planes.h pgm.h png_pov.h ppm.h prism.h radiosit.h sor.h spheres.h super.h targa.h \
		texture.h pattern.h tokenize.h torus.h triangle.h truetype.h userio.h lbuffer.h \
		vbuffer.h 

port.o:		port.c frame.h config.h povproto.h mem.h povray.h atmosph.h warps.h \
		camera.h point.h vlbuffer.h bbox.h render.h optin.h 

polysolv.o:	polysolv.c frame.h config.h povray.h atmosph.h warps.h camera.h \
		point.h vlbuffer.h bbox.h render.h povproto.h mem.h vector.h polysolv.h 

polygon.o:	polygon.c frame.h config.h vector.h povproto.h mem.h matrices.h \
		objects.h polygon.h povray.h atmosph.h warps.h camera.h point.h vlbuffer.h bbox.h \
		render.h 

poly.o:		poly.c frame.h config.h vector.h povproto.h mem.h bbox.h polysolv.h \
		matrices.h objects.h poly.h povray.h atmosph.h warps.h camera.h point.h vlbuffer.h \
		render.h 

point.o:	point.c frame.h config.h vector.h povproto.h mem.h point.h vlbuffer.h \
		bbox.h matrices.h objects.h povray.h atmosph.h warps.h camera.h render.h 

png_pov.o:	png_pov.c frame.h config.h povproto.h mem.h povray.h atmosph.h warps.h \
		camera.h point.h vlbuffer.h bbox.h render.h optout.h png_pov.h 

planes.o:	planes.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h matrices.h objects.h planes.h 

pigment.o:	pigment.c frame.h config.h vector.h povproto.h mem.h texture.h pattern.h \
		warps.h colour.h image.h matrices.h pigment.h txttest.h 

pgm.o:		pgm.c frame.h config.h povproto.h mem.h povray.h atmosph.h warps.h camera.h \
		point.h vlbuffer.h bbox.h render.h pgm.h ppm.h 

pattern.o:	pattern.c frame.h config.h vector.h povproto.h mem.h matrices.h \
		pattern.h texture.h warps.h image.h txttest.h colour.h 

parstxtr.o:	parstxtr.c frame.h config.h vector.h povproto.h mem.h parse.h parstxtr.h \
		atmosph.h warps.h colour.h express.h gif.h halos.h iff.h image.h matrices.h \
		normal.h pigment.h povray.h camera.h point.h vlbuffer.h bbox.h render.h pgm.h \
		ppm.h targa.h png_pov.h texture.h pattern.h tokenize.h 

parse.o:	parse.c frame.h config.h vector.h povproto.h mem.h parse.h parstxtr.h \
		atmosph.h warps.h bezier.h blob.h bsphere.h boxes.h colour.h cones.h csg.h discs.h \
		express.h fractal.h gif.h halos.h hfield.h bbox.h iff.h image.h lathe.h polysolv.h \
		matrices.h mesh.h normal.h objects.h octree.h pigment.h planes.h poly.h polygon.h \
		povray.h camera.h point.h vlbuffer.h render.h pgm.h ppm.h prism.h quadrics.h \
		radiosit.h sor.h spheres.h super.h targa.h texture.h pattern.h tokenize.h torus.h \
		triangle.h truetype.h 

optout.o:	optout.c frame.h config.h vector.h povproto.h mem.h atmosph.h warps.h \
		bezier.h blob.h bsphere.h bbox.h cones.h csg.h discs.h fractal.h hfield.h boxes.h \
		lathe.h lighting.h point.h vlbuffer.h mesh.h polysolv.h objects.h parse.h poly.h \
		polygon.h octree.h quadrics.h planes.h pgm.h ppm.h prism.h radiosit.h render.h \
		sor.h spheres.h super.h targa.h texture.h pattern.h torus.h triangle.h truetype.h \
		userio.h lbuffer.h vbuffer.h povray.h camera.h optin.h optout.h 

optin.o:	optin.c frame.h config.h povproto.h mem.h bbox.h lighting.h point.h \
		vlbuffer.h octree.h povray.h atmosph.h warps.h camera.h render.h optin.h optout.h \
		parse.h radiosit.h tokenize.h ppm.h targa.h userio.h png_pov.h 

octree.o:	octree.c frame.h config.h vector.h povproto.h mem.h povray.h atmosph.h \
		warps.h camera.h point.h vlbuffer.h bbox.h render.h octree.h radiosit.h 

objects.o:	objects.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h objects.h texture.h pattern.h \
		halos.h 

normal.o:	normal.c frame.h config.h vector.h povproto.h mem.h texture.h pattern.h \
		warps.h image.h matrices.h normal.h povray.h atmosph.h camera.h point.h vlbuffer.h \
		bbox.h render.h txttest.h pigment.h 

mesh.o:		mesh.c frame.h config.h vector.h povproto.h mem.h bbox.h matrices.h \
		objects.h mesh.h texture.h pattern.h warps.h povray.h atmosph.h camera.h point.h \
		vlbuffer.h render.h 

mem.o:		mem.c frame.h config.h povproto.h mem.h parse.h povray.h atmosph.h warps.h \
		camera.h point.h vlbuffer.h bbox.h render.h 

matrices.o:	matrices.c frame.h config.h vector.h povproto.h mem.h matrices.h 

lighting.o:	lighting.c frame.h config.h vector.h povproto.h mem.h blob.h bsphere.h \
		bbox.h colour.h halos.h image.h lbuffer.h vlbuffer.h lighting.h point.h mesh.h \
		normal.h objects.h octree.h pattern.h pigment.h povray.h atmosph.h warps.h camera.h \
		render.h radiosit.h ray.h texture.h 

lbuffer.o:	lbuffer.c frame.h config.h vector.h povproto.h mem.h point.h vlbuffer.h \
		bbox.h povray.h atmosph.h warps.h camera.h render.h lbuffer.h objects.h triangle.h 

lathe.o:	lathe.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h lathe.h polysolv.h matrices.h \
		objects.h torus.h 

image.o:	image.c frame.h config.h vector.h povproto.h mem.h texture.h pattern.h \
		warps.h image.h matrices.h povray.h atmosph.h camera.h point.h vlbuffer.h bbox.h \
		render.h 

iff.o:		iff.c frame.h config.h povproto.h mem.h iff.h povray.h atmosph.h warps.h \
		camera.h point.h vlbuffer.h bbox.h render.h 

hfield.o:	hfield.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h hfield.h boxes.h matrices.h \
		objects.h 

hcmplx.o:	hcmplx.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h fractal.h spheres.h hcmplx.h 

halos.o:	halos.c frame.h config.h povproto.h mem.h vector.h povray.h atmosph.h \
		warps.h camera.h point.h vlbuffer.h bbox.h render.h pattern.h texture.h halos.h \
		objects.h matrices.h lighting.h colour.h 

gifdecod.o:	gifdecod.c frame.h config.h povproto.h mem.h gif.h gifdecod.h 

gif.o:		gif.c frame.h config.h povproto.h mem.h gif.h gifdecod.h povray.h atmosph.h \
		warps.h camera.h point.h vlbuffer.h bbox.h render.h 

fractal.o:	fractal.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h matrices.h objects.h spheres.h \
		fractal.h quatern.h hcmplx.h 

express.o:	express.c frame.h config.h vector.h povproto.h mem.h parse.h parstxtr.h \
		atmosph.h warps.h colour.h express.h matrices.h povray.h camera.h point.h vlbuffer.h \
		bbox.h render.h tokenize.h pattern.h pigment.h normal.h texture.h 

discs.o:	discs.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h discs.h matrices.h objects.h 

csg.o:		csg.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h vlbuffer.h \
		bbox.h render.h vector.h povproto.h mem.h csg.h hfield.h boxes.h matrices.h \
		objects.h planes.h quadrics.h 

cones.o:	cones.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h cones.h matrices.h objects.h 

colour.o:	colour.c frame.h config.h vector.h povproto.h mem.h colour.h pigment.h \
		normal.h texture.h pattern.h warps.h 

chi2.o:		chi2.c frame.h config.h povproto.h mem.h chi2.h 

camera.o:	camera.c frame.h config.h vector.h povproto.h mem.h camera.h matrices.h \
		normal.h 

bsphere.o:	bsphere.c frame.h config.h vector.h povproto.h mem.h bsphere.h 

boxes.o:	boxes.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h boxes.h matrices.h objects.h 

blob.o:		blob.c frame.h config.h povray.h atmosph.h warps.h camera.h point.h \
		vlbuffer.h bbox.h render.h vector.h povproto.h mem.h blob.h bsphere.h lighting.h \
		matrices.h objects.h polysolv.h texture.h pattern.h 

bezier.o:	bezier.c frame.h config.h vector.h povproto.h mem.h bezier.h matrices.h \
		objects.h povray.h atmosph.h warps.h camera.h point.h vlbuffer.h bbox.h render.h 

bbox.o:		bbox.c frame.h config.h vector.h povproto.h mem.h bbox.h matrices.h \
		objects.h povray.h atmosph.h warps.h camera.h point.h vlbuffer.h render.h 

atmosph.o:	atmosph.c frame.h config.h vector.h povproto.h mem.h atmosph.h warps.h \
		colour.h povray.h camera.h point.h vlbuffer.h bbox.h render.h texture.h pattern.h \
		pigment.h objects.h lighting.h matrices.h

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		*.C rules							#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

.SUFFIXES:	.o .c

.c.o:
	$(CC)	$*.c -c $(CMODE) $(CTARG) $(COPTI) -o$*.o

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#		*.S rules (GEN)							#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

.SUFFIXES:	.a .s

.s.o:
	$(GEN)	$*.o $*.s
	gst2gnu $*.o

